<?php
namespace App\Http\Controllers\Api;

use App\Http\Controllers\BaseController;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Throwable;
use Exception;
use Carbon\Carbon;
use Illuminate\Validation\Rule;
use App\Services\White\IWhiteService;

class WhiteController extends BaseController
{
    private $time;
    private $whiteService;

    public function __construct(IWhiteService $whiteService)
    {
        $this->time = Carbon::now()->format('Y-m-d H:i:s');
        $this->whiteService = $whiteService;
    }

    /**
     * 新增白名单
     * @return array
     * @date 2021/07/29
     * @author live
     */
    public function getWhiteAdd(Request $request)
    {
        $postData = $request->all();
        $validate = Validator::make($postData, [
            'name' => 'required|string',
            'uid' => 'required|string'
        ]);
        if ($validate->fails()) return $this->getErrorMsg('1001', config('msg.common.1001'));
        try {
            $data = $this->whiteService->getWhiteAdd($postData);
            return $this->getSuccessMsg($data);
        } catch (Exception $e) {
            return $this->getErrorMsg($e->getCode(), $e->getMessage());
        }
    }

    /**
     * 白名单列表
     * @return array
     * @date 2021/07/29
     * @author live
     */
    public function getWhiteList(Request $request)
    {
        $postData = $request->all();
        try {
            $data = $this->whiteService->getWhiteList($postData);
            return $this->getSuccessMsg($data);
        } catch (Exception $e) {
            return $this->getErrorMsg($e->getCode(), $e->getMessage());
        }
    }

    /**
     * 删除白名单
     * @return array
     * @date 2021/07/29
     * @author live
     */
    public function getWhiteDel(Request $request)
    {
        $postData = $request->all();
        $validate = Validator::make($postData, [
            'id' => 'required|numeric'
        ]);

        if ($validate->fails()) return $this->getErrorMsg('1001', config('msg.common.1001'));
        try {
            $data = $this->whiteService->getWhiteDel($postData);
            return $this->getSuccessMsg($data);
        } catch (Exception $e) {
            return $this->getErrorMsg($e->getCode(), $e->getMessage());
        }
    }

    /**
     * 设置白名单状态
     * @return array
     * @date 2021/07/29
     * @author live
     */
    public function getSetStatus(Request $request)
    {
        $postData = $request->all();
        $validate = Validator::make($postData, [
            'id' => 'required|numeric',
            'status' => 'required|numeric'
        ]);
        if ($validate->fails()) return $this->getErrorMsg('1001', config('msg.common.1001'));

        try {
            $data = $this->whiteService->getSetStatus($postData);
            return $this->getSuccessMsg($data);
        } catch (Exception $e) {
            return $this->getErrorMsg($e->getCode(), $e->getMessage());
        }
    }


}