<?php
namespace App\Services\Account;

use App\Services\Account\IAccountService;
use Carbon\Carbon;
use Exception;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Str;
use App\Models\SyceeLogModel;
use App\Models\AccountModel;
use Illuminate\Support\Facades\Cache;

class AccountService implements IAccountService
{
    private $time;

    public function __construct()
    {
        $this->time = Carbon::now()->format('Y-m-d H:i:s');
    }

    /**
     * 平台列表
     * @return araay
     * @date 2021/07/30
     * @author live
     */
    public function getPlatformList($postData)
    {
        $array = ['0'=>'请选择'];
        foreach(config(env('ENVIRONMENT') . '.PLATFORM') as $key=>$val) $array[$key] = $val;
        return $array;
    }

    /**
     * 渠道列表
     * @return araay
     * @date 2021/07/30
     * @author live
     */
    public function getChannelList($postData)
    {
        $array = ['0'=>'请选择'];
        foreach(config(env('ENVIRONMENT') . '.CHANNEL') as $key=>$val) $array[$key] = $val;
        return $array;
    }


    /**
     * 货币类型列表
     * @return araay
     * @date 2021/07/30
     * @author live
     */
    public function getCurrencyList($postData)
    {
        $array = ['0'=>'请选择'];
        foreach(config(env('ENVIRONMENT') . '.CURRENCY') as $key=>$val) $array[$key] = $val;
        return $array;
    }

    /**
     * 货币查询
     * @return araay
     * @date 2021/07/30
     * @author live
     */
    public function getAccountLog($postData)
    {
        $pagePerNum = $postData['pageSize'] ?? 20;
        $currentPage = $postData['page'] ?? 1;
        $start = $postData['startTime'] ?? '';
        $end = $postData['endTime'] ?? '';
        $platform = $postData['platform'] ?? 0;
        $channel = $postData['channel'] ?? 0;
        $logType = $postData['logType'] ?? 0;
        $type = $postData['type'] ?? 0;
        $uid = $postData['uid'] ?? '';
        if($uid == '') return [];
        $account = AccountModel::where(function ($query) use ($uid) {
            return $query->where('user_id',$uid)->orWhere('platform_id',$uid);
        })->when($platform > 0, function ($query) use ($platform) {
            return $query->where('platform', $platform);
        })->when($channel > 0, function ($query) use ($channel) {
            return $query->where('channel', $channel);
        })->select('user_id','sycee','platform','platform_id','channel')->first();
        if(!$account) return [];

        $array = [];
        $logs = SyceeLogModel::where('user_id',$account->user_id)->when($start != '', function ($query) use ($start) {
            $startTime = date('Y-m-d H:i:s',strtotime($start) - 1);
            return $query->where('log_time', '>',$startTime);
        })->when($end != '', function ($query) use ($end) {
            $endTime = date('Y-m-d H:i:s',strtotime($end) + 86401);
            return $query->where('log_time', '<',$endTime);
        })->when($logType > 0, function ($query) use ($logType) {
            if($logType == 1) return $query->whereIn('op_code',[1,2,4]);
            if($logType == 2) return $query->where('op_code',3);
        })->select('id','op_code','sycee','sycee_pool','sycee_balance','sycee_pool_balance','log_time','extra')->orderBy('id','DESC')->paginate($pagePerNum, ['*'], 'currentPage',$currentPage);
        $array['totalNum'] = $logs->total();
        $array['totalPage'] = $logs->lastPage();
        $platformArr = config(env('ENVIRONMENT') . '.PLATFORM');
        $channelArr = config(env('ENVIRONMENT') . '.CHANNEL');
        $sourceArr = SyceeLogModel::$source;
        foreach($logs as $log){
            $tem['uid'] = $account->platform_id;
            $tem['openId'] = $account->user_id;
            $tem['platformName'] = $platformArr[$account->platform] ?? '';
            $tem['channelName'] = $channelArr[$account->channel] ?? '';
            $tem['typeName'] = '';
            if($log->op_code == 1) $tem['typeName'] = '产出(创角)';
            if($log->op_code == 2) $tem['typeName'] = '产出(观看广告)';
            if($log->op_code == 3) $tem['typeName'] = '消耗(元宝提现)';
            if($log->op_code == 4) $tem['typeName'] = '产出(提现失败返还)';
            $tem['beforeNum'] = $log->sycee_balance - $log->sycee;
            $tem['optionNum'] = $log->sycee;
            $tem['afterNum'] = $log->sycee_balance;
            $tem['source'] = $sourceArr[$log->op_code];
            $tem['logTime'] = $log->log_time;
            $tem['remark'] = $log->extra;
            $array['data'][] = $tem;
        }
        unset($logs,$log,$account);
        return $array;
    }
}