<?php
namespace App\Services\Warn;

use App\Services\Warn\IWarnService;
use Carbon\Carbon;
use Exception;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Str;
use App\Models\WarnPeopleModel;
use App\Models\WarnConfigModel;
use App\Models\WarnLogModel;
use Illuminate\Support\Facades\Cache;

class WarnService implements IWarnService
{
    private $time;

    public function __construct()
    {
        $this->time = Carbon::now()->format('Y-m-d H:i:s');
    }

    /**
     * 新增预警人信息
     * @return int
     * @date 2021/07/31
     * @author live
     */
    public function getPeopleAdd($postData)
    {
        if(!preg_match("/^1[3456789]\d{9}$/", $postData['phone'])) throw new Exception(config('msg.common.1013'), 1013);
        if($postData['platform'] < 1 || $postData['channel'] < 1) throw new Exception(config('msg.common.1014'), 1014);
        $verity = WarnPeopleModel::where('platform', $postData['platform'])->where('channel', $postData['channel'])->where('phone',$postData['phone'])->select('id')->first();
        if (!$verity) {
            $row = new WarnPeopleModel();
            $row->platform = $postData['platform'] ?? 0;
            $row->channel = $postData['channel'] ?? 0;
            $row->name = $postData['remarkName'] ?? '';
            $row->phone = $postData['phone'] ?? '';
            $row->op_user = $postData['user'] ?? '';
            $row->remark =  $postData['remark'] ?? '';
            if ($row->save()) return $row->id;
            throw new Exception(config('msg.common.1006'), 1006);
        }
        throw new Exception(config('msg.common.1005'), 1005);
    }


    /**
     * 预警人列表
     * @return araay
     * @date 2021/07/31
     * @author live
     */
    public function getPeopleList($postData)
    {
        $name = $postData['key'] ?? '';
        $platform = $postData['platform'] ?? 0;
        $start = $postData['startTime'] ?? '';
        $end = $postData['endTime'] ?? '';
        $pagePerNum = $postData['pageSize'] ?? 20;
        $currentPage = $postData['page'] ?? 1;

        $lists = WarnPeopleModel::when($name != '', function ($query) use ($name) {
            return $query->where('name', 'like', $name . '%');
        })->when($platform > 0, function ($query) use ($platform) {
            return $query->where('platform',$platform);
        })->when($start != '', function ($query) use ($start) {
            $startTime = date('Y-m-d H:i:s',strtotime($start) - 1);
            return $query->where('created_at', '>',$startTime);
        })->when($end != '', function ($query) use ($end) {
            $endTime = date('Y-m-d H:i:s',strtotime($end) + 86401);
            return $query->where('created_at', '<',$endTime);
        })->select('id', 'platform', 'channel', 'name', 'phone','status','op_user', 'created_at')->orderBy('id', 'DESC')->paginate($pagePerNum, ['*'], 'currentPage', $currentPage);
        $array['totalNum'] = $lists->total();
        $array['totalPage'] = $lists->lastPage();
        $platformArr = config(env('ENVIRONMENT') . '.PLATFORM');
        $channelArr = config(env('ENVIRONMENT') . '.CHANNEL');

        foreach ($lists as $list) {
            $tem['id'] = $list->id;
            $tem['platform'] = $platformArr[$list->platform] ?? '';
            $tem['platformId'] = $list->platform;
            $tem['channel'] = $channelArr[$list->channel] ?? '';
            $tem['channelId'] = $list->channel;
            $tem['name'] = $list->name;
            $tem['phone'] = $list->phone;
            $tem['status'] = $list->status;
            $statusName = '停用';
            if($list->status == 1) $statusName = '正常';
            $tem['statusName'] = $statusName;
            $tem['optionUser'] = $list->op_user;
            $tem['createdAt'] = Carbon::parse($list->created_at)->format('Y-m-d H:i:s');
            $array['data'][] = $tem;
        }
        unset($lists, $list);
        return $array;
    }

    /**
     * 编辑预警人
     * @return araay
     * @date 2021/07/31
     * @author live
     */
    public function getPeopleEdit($postData)
    {
        if(!preg_match("/^1[3456789]\d{9}$/", $postData['phone'])) throw new Exception(config('msg.common.1013'), 1013);
        if($postData['platform'] < 1 || $postData['channel'] < 1) throw new Exception(config('msg.common.1014'), 1014);
        $verity = WarnPeopleModel::where('id','!=',$postData['id'])->where('platform', $postData['platform'])->where('channel', $postData['channel'])->where('phone',$postData['phone'])->select('id')->first();
        if (!$verity) {
            $row = WarnPeopleModel::find($postData['id']);
            if(!$row) throw new Exception(config('msg.common.1002'), 1002);
            $row->platform = $postData['platform'] ?? 0;
            $row->channel = $postData['channel'] ?? 0;
            $row->name = $postData['remarkName'] ?? '';
            $row->phone = $postData['phone'] ?? '';
            $row->op_user = $postData['user'] ?? '';
            $row->remark =  $postData['remark'] ?? '';
            if ($row->save()) return $row->id;
            throw new Exception(config('msg.common.1006'), 1006);
        }
        throw new Exception(config('msg.common.1005'), 1005);
    }


    /**
     * 删除预警人
     * @return araay
     * @date 2021/07/31
     * @author live
     */
    public function getPeopleDel($postData)
    {
        $list = WarnPeopleModel::where('id', $postData['id'])->first();
        if (!$list) throw new Exception(config('msg.common.1002'), 1002);
        if ($list->delete()) return true;
        throw new Exception(config('msg.common.1003'), 1003);
    }

    /**
     * 设置预警人状态
     * @return araay
     * @date 2021/07/31
     * @author live
     */
    public function getPeopleSetStatus($postData)
    {
        $list = WarnPeopleModel::where('id', $postData['id'])->first();
        if (!$list) throw new Exception(config('msg.common.1002'), 1002);

        if($list->status == $postData['status']) throw new Exception(config('msg.common.1009'), 1009);
        $list->status = $postData['status'];
        $list->op_user = $postData['user'] ?? '';
        $list->updated_at =  $this->time;
        if ($list->save()) return true;
        throw new Exception(config('msg.common.1003'), 1003);
    }

    /**
     * 预警配置添加
     * @return araay
     * @date 2021/07/31
     * @author live
     */
    public function getConfigAdd($postData)
    {
        if($postData['platform'] < 1 || $postData['channel'] < 1 || $postData['type'] < 1) throw new Exception(config('msg.common.1014'), 1014);
        $verity = WarnConfigModel::where('platform', $postData['platform'])->where('channel', $postData['channel'])->where('type',$postData['type'])->select('id')->first();
        if (!$verity) {
            $row = new WarnConfigModel();
            $row->platform = $postData['platform'] ?? 0;
            $row->channel = $postData['channel'] ?? 0;
            $row->type = $postData['type'] ?? 0;
            $row->one_withdrawal = $postData['oneWithdrawal'] * 100;
            $row->day_withdrawal = $postData['dayWithdrawal']  * 100;
            $row->people_day_withdrawal = $postData['userDayWithdrawal'] * 100;
            $row->op_user = $postData['user'] ?? '';
            $row->remark =  $postData['remark'] ?? '';
            if ($row->save()) return $row->id;
            throw new Exception(config('msg.common.1006'), 1006);
        }
        throw new Exception(config('msg.common.1016'), 1016);
    }

    /**
     * 预警配置列表
     * @return araay
     * @date 2021/07/31
     * @author live
     */
    public function getConfigList($postData)
    {
        $status = $postData['status'] ?? 0;
        $pagePerNum = $postData['pageSize'] ?? 20;
        $currentPage = $postData['page'] ?? 1;

        $lists = WarnConfigModel::when($status > 0, function ($query) use ($status) {
            return $query->where('status',$status);
        })->select('id', 'platform', 'channel', 'type', 'one_withdrawal','day_withdrawal','people_day_withdrawal','status','op_user', 'created_at')->orderBy('id', 'DESC')->paginate($pagePerNum, ['*'], 'currentPage', $currentPage);
        $array['totalNum'] = $lists->total();
        $array['totalPage'] = $lists->lastPage();
        $platformArr = config(env('ENVIRONMENT') . '.PLATFORM');
        $channelArr = config(env('ENVIRONMENT') . '.CHANNEL');
        $typeArr = config(env('ENVIRONMENT') . '.CURRENCY');

        foreach ($lists as $list) {
            $tem['id'] = $list->id;
            $tem['platform'] = $platformArr[$list->platform] ?? '';
            $tem['platformId'] = $list->platform;
            $tem['channel'] = $channelArr[$list->channel] ?? '';
            $tem['channelId'] = $list->channel;
            $tem['type'] = $typeArr[$list->type] ?? '';
            $tem['typeId'] = $list->type;
            $tem['oneWithdrawal'] = sprintf('%.2f',$list->one_withdrawal / 100);
            $tem['dayWithdrawal'] = sprintf('%.2f',$list->day_withdrawal / 100);
            $tem['userDayWithdrawal'] = sprintf('%.2f',$list->people_day_withdrawal / 100);
            $tem['status'] = $list->status;
            $statusName = '停用';
            if($list->status == 1) $statusName = '正常';
            $tem['statusName'] = $statusName;
            $tem['optionUser'] = $list->op_user;
            $tem['createdAt'] = Carbon::parse($list->created_at)->format('Y-m-d H:i:s');
            $array['data'][] = $tem;
        }
        unset($lists, $list);
        return $array;
    }

    /**
     * 预警配置编辑
     * @return araay
     * @date 2021/07/31
     * @author live
     */
    public function getConfigEdit($postData)
    {
        if($postData['platform'] < 1 || $postData['channel'] < 1 || $postData['type'] < 1) throw new Exception(config('msg.common.1014'), 1014);
        $verity = WarnConfigModel::where('id','!=',$postData['id'])->where('platform', $postData['platform'])->where('channel', $postData['channel'])->where('type',$postData['type'])->where('one_withdrawal',$postData['oneWithdrawal'])->where('day_withdrawal',$postData['dayWithdrawal'])->where('people_day_withdrawal',$postData['userDayWithdrawal'])->select('id')->first();
        if (!$verity) {
            $row = WarnConfigModel::find($postData['id']);
            $row->platform = $postData['platform'] ?? 0;
            $row->channel = $postData['channel'] ?? 0;
            $row->type = $postData['type'] ?? 0;
            $row->one_withdrawal = $postData['oneWithdrawal'] * 100;
            $row->day_withdrawal = $postData['dayWithdrawal']  * 100;
            $row->people_day_withdrawal = $postData['userDayWithdrawal'] * 100;
            $row->op_user = $postData['user'] ?? '';
            $row->remark =  $postData['remark'] ?? '';
            $row->updated_at = $this->time;
            if ($row->save()) return $row->id;
            throw new Exception(config('msg.common.1006'), 1006);
        }
        throw new Exception(config('msg.common.1005'), 1005);
    }

    /**
     * 预警配置删除
     * @return araay
     * @date 2021/07/31
     * @author live
     */
    public function getConfigDel($postData)
    {
        $list = WarnConfigModel::where('id', $postData['id'])->first();
        if (!$list) throw new Exception(config('msg.common.1002'), 1002);
        if ($list->delete()) return true;
        throw new Exception(config('msg.common.1003'), 1003);
    }


    /**
     * 预警配置状态设置
     * @return araay
     * @date 2021/07/31
     * @author live
     */
    public function getConfigSetStatus($postData)
    {
        $list = WarnConfigModel::where('id', $postData['id'])->first();
        if (!$list) throw new Exception(config('msg.common.1002'), 1002);

        if($list->status == $postData['status']) throw new Exception(config('msg.common.1009'), 1009);
        $list->status = $postData['status'];
        $list->op_user = $postData['user'] ?? '';
        $list->updated_at =  $this->time;
        if ($list->save()) return true;
        throw new Exception(config('msg.common.1003'), 1003);
    }

    /**
     * 批量设置预警状态
     * @return araay
     * @date 2021/07/31
     * @author live
     */
    public function getConfigSetAllStatus($postData)
    {
        $array = ['list'=>true,'msg'=>''];
        $ids = explode('#',$postData['ids']);
        if(empty($ids)) throw new Exception(config('msg.common.1015'), 1015);
        $lists = WarnConfigModel::whereIn('id', $ids)->get();
        if ($lists->isEmpty()) throw new Exception(config('msg.common.1002'), 1002);
        $total = $lists->count();
        $succuss = 0;
        $fail = 0;
        foreach($lists as $list){
            $list->status = $postData['status'];
            $list->op_user = $postData['user'] ?? '';
            $list->updated_at =  $this->time;
            if ($list->save()){
                $succuss++;
            }else{
                $fail++;
            }
        }
        $array['msg'] = '总共操作'.$total.'条数据，其中成功'.$succuss.'条，失败'.$fail.'条';
        return $array;
    }

    /**
     * 预警日志
     * @return araay
     * @date 2021/07/31
     * @author live
     */
    public function getWarnLog($postData)
    {
        $start = $postData['startTime'] ?? '';
        $end = $postData['endTime'] ?? '';
        $platform = $postData['platform'] ?? 0;
        $id = $postData['id'] ?? 0;
        $pagePerNum = $postData['pageSize'] ?? 20;
        $currentPage = $postData['page'] ?? 1;

        $lists = WarnLogModel::when($id > 0, function ($query) use ($id) {
            return $query->where('id',$id);
        })->when($platform > 0, function ($query) use ($platform) {
            return $query->where('platform',$platform);
        })->when($start > 0, function ($query) use ($start) {
            $startTime = date('Y-m-d H:i:s',strtotime($start) - 1);
            return $query->where('send_time','>',$startTime);
        })->when($end > 0, function ($query) use ($end) {
            $endTime = date('Y-m-d H:i:s',strtotime($end) + 86401);
            return $query->where('send_time','<',$endTime);
        })->select('id','platform','channel','cash','status','type','config_id','people_id','msg','remark','send_time','created_at')->orderBy('id', 'DESC')->paginate($pagePerNum, ['*'], 'currentPage', $currentPage);
        $array['totalNum'] = $lists->total();
        $array['totalPage'] = $lists->lastPage();
        $platformArr = config(env('ENVIRONMENT') . '.PLATFORM');
        $channelArr = config(env('ENVIRONMENT') . '.CHANNEL');

        $idArr = [];
        foreach ($lists as $list) {
            $tem['id'] = $list->id;
            $tem['platform'] = $platformArr[$list->platform] ?? '';
            $tem['channel'] = $channelArr[$list->channel] ?? '';
            $tem['amount'] = sprintf('%.2f',$list->cash / 100);
            $tem['oneWithdrawal'] = 0.00;
            $tem['dayWithdrawal'] = 0.00;
            $tem['userDayWithdrawal'] = 0.00;
            $tem['configId'] = $list->config_id;
            $tem['peopleId'] = $list->people_id;
            $tem['status'] = $list->status;
            $statusName = '已发送';
            if($list->status == 0) $statusName = '未发送';
            $tem['statusName'] = $statusName;
            $tem['msg'] = $list->msg;
            $tem['createdAt'] = $list->send_time;
            $array['data'][] = $tem;
            $idArr[$list->config_id] = $list->config_id;
        }

        if(!empty($idArr)){
            $conArr = [];
            $configs = WarnConfigModel::whereIn('id',$idArr)->select('id','one_withdrawal','day_withdrawal','people_day_withdrawal')->get();
            foreach($configs as $con){
                $ctem['oneWithdrawal'] = sprintf('%.2f',$con->one_withdrawal / 100);
                $ctem['dayWithdrawal'] = sprintf('%.2f',$con->day_withdrawal / 100);
                $ctem['userDayWithdrawal'] = sprintf('%.2f',$con->people_day_withdrawal / 100);
                $conArr[$con->id] = $ctem;
            }

            if(!empty($conArr)){
                foreach($array['data'] as &$val){
                    $val['oneWithdrawal'] = $conArr[$val['configId']]['oneWithdrawal'] ?? '0.00';
                    $val['dayWithdrawal'] = $conArr[$val['configId']]['dayWithdrawal'] ?? '0.00';
                    $val['userDayWithdrawal'] = $conArr[$val['configId']]['userDayWithdrawal'] ?? '0.00';
                }
            }
        }

        unset($lists, $list);
        return $array;
    }
}